const { exec } = require('child_process')
const chalk = require('chalk')
const inquirer = require('inquirer')
const notify = require('./notify')

module.exports = function(module, next = () => {}, args = {}) {
  const {updateAll} = args

  if (updateAll === false)
    return next()

  notify.info(`Check module '${module}' version...`)

  exec(`npm view ${module} version`, {timeout: 3000}, (error, version, stderr) => {
    if (!error) {
      try {var localVersion = require(`${module}/package.json`).version}
      catch(e) {
        notify.warn(`Cannot find module '${module}'`)
        notify.info(`Try to install the module '${module}', please wait a moment...`)
        return update(module, next)
      }

      // 去除换行符
      version = version.trim()

      if (version !== localVersion) {
        const updateHandle = () => {
          notify.info('Updating...')
          update(module, next)
        }

        if (updateAll === undefined)
          inquirer.prompt([{
            type: 'confirm',
            name: 'willUpdate',
            message: `Module '${chalk.cyan(module)}' detects the new version. Is it updated?(${chalk.blue(`local: v${localVersion}`)}, ${chalk.green(`origin: v${version}`)})`
          }]).then(({ willUpdate }) => !willUpdate ? next() : updateHandle())

        updateAll && updateHandle()
      } else next()
    } else {
      notify.warn(`Module '${module}' version check failed`)
      next(error)
    }
  })
}

function update(module, callback) {
  exec(`npm install -g ${module} --unsafe-perm --sass_binary_site=https://npm.taobao.org/mirrors/node-sass/`, (error, stdout, stderr) => {
    console.log(stdout)
    console.log(stderr)

    if (error) notify.warn(`Module '${module}' update failed, try to using sudo permission in the next time.`)
    else notify.success(`Module '${module}' has been updated.`)

    callback(error)
  })
}